unit FileToolbar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TB97, TB97Tlbr, TB97Ctls, ActnList;

type
  TFileToolbar = class(TToolbar97)
  private
   FAction:   TActionList;
   FImages:   TImageList;
   FOnSave:   TNotifyEvent;
   FOnLoad:   TNotifyEvent;
   FOriginal: TStringList;
   FUsereg:   Boolean;
   function GetAction (const Name: string): TAction;
   procedure Store (sl: TStringList);
   procedure Reset (sl: TStringList; Clear: Boolean);
    { Private-Deklarationen }
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
  published
    { Published-Deklarationen }
   constructor Create (AOwner: TComponent); override;
   destructor  Destroy; override;
   property Actions: TActionList read FAction Write FAction;
   property Images: TImageList read FImages Write FImages;
   property OnSave: TNotifyEvent read FOnSave Write FOnSave;
   property OnLoad: TNotifyEvent read FOnLoad Write FOnSave;
   property UseRegistry: Boolean read FUsereg Write FUsereg;
   procedure StoreToolbar;
   procedure ResetToolbar;
   procedure LoadFromfile (const fName: string; clear: Boolean);
   procedure SaveToFile (const fName: string);
  end;

procedure Register;

implementation

uses IniFiles, Registry;

const cGTAction =  '.GT_Action';
      cGAction =   '.G_Action';
      cTAction =   '.T_Action';
      cSeparator = '.Separator';
      cCaption =   'Caption';


procedure Register;
begin
  RegisterComponents('Toolbar97', [TFileToolbar]);
end;


constructor TFileToolbar.Create (AOwner: TComponent);
begin
 inherited;
 FAction:= nil;
 FImages:= nil;
 FOnSave:= nil;
 FOnLoad:= nil;
 FOriginal:= nil;
 FUsereg:= False;
 FOriginal:= TStringList.Create;
end;


destructor TFileToolbar.Destroy;
begin
 FOriginal.Free;
 inherited;
end;


function TFileToolbar.GetAction (const Name: string): TAction;
// Search Name in the ToolBars ActionList and return the Action
var i: integer;
    act: TCustomAction;
    s1, s2: string;
begin
 Result:= nil;
 if FAction <> nil then begin
  s1:= UpperCase (Name);
  for i:= 0 to FAction.ActionCount - 1 do begin
   act:= TCustomAction (FAction.Actions[i]);
   if act <> nil then begin
    s2:= UpperCase (act.Name);
    if s1 = s2 then begin
     Result:= act as TAction;
     Exit;
    end;
   end;
  end;
 end;
end;


procedure TFileToolbar.Store (sl: TStringList);
var i, x: integer;
    s: string;
    tmp: TControl;
    tb: TToolbarButton97;
    ts: TToolbarSep97;
begin
 if Assigned (sl) then begin
  sl.Add (cCaption + '=' + Caption);

  for i:= 0 to ControlCount - 1 do begin
   // Loop through OrderIndex to write the correct Order to the file

   for x:= 0 to ControlCount - 1 do begin
    tmp:= Controls[x];

    // Look about controls orderindex...
    if OrderIndex[tmp] = i then begin

     // Toolbarbottons
     if (tmp is TToolbarButton97) then begin
      tb:= (tmp as TToolbarButton97);
      case tb.DisplayMode of
       dmBoth:     s:= cGTAction;
       dmTextOnly: s:= cTAction;
       else        s:= cGAction;
      end;

      sl.Add (tb.Name + s + '=' + tb.Action.Name);
     end; // if (tmp is TToolbarButton97)

     // Separator
     if (tmp is TToolbarSep97) then begin
      ts:= (tmp as TToolbarSep97);
      sl.Add (ts.Name + cSeparator + '=X');
     end; // if (tmp is TToolbarSep97)

    end; // if OrderIndex[tmp]
   end; // for x
  end; // for y
 end; // if Assigned
end; // Store


procedure TFileToolbar.Reset (sl: TStringList; Clear: Boolean);
var i, x: integer;
    tmp: TControl;
    ts: TToolbarSep97;
    s1, s2, cn: string;

 procedure DelControl (const s: string);
 // Removes the control with name "s" from the Toolbar
 var i: integer;
 begin
  for i:= ControlCount - 1 downto 0 do begin
   tmp:= Controls[i];
   if tmp.Name = s then begin
    tmp.Free;
    Exit;
   end;
  end;
 end;

 procedure GenButton (const bName, bAction: string; bmode: TButtonDisplayMode);
 // Generate a new TToolbarButton97
 var tb: TToolbarButton97;
 begin
  DelControl (bName); // Remove old control with this name
  tb:= TToolbarButton97.Create (Self); // Create new one
  tb.Name:= bName;
  tb.Images:= FImages; // Link to the Imagelist
  tb.Action:= GetAction (bAction); // Find the Action
  tb.DisplayMode:= bMode; // DisplayMode
  InsertControl (tb); // Insert the new button
 end;

begin
 if sl.Count > 0 then begin

  BeginUpdate; // Stop flickering

  if Clear then begin
   for i:= ControlCount - 1 downto 0 do begin
    Controls[i].Free;
   end;
  end;

  // Generate the controls
  for i:= 0 to sl.Count - 1 do begin
   s1:= sl.Names[i];
   s2:= sl.Values[s1];

   // set caption of the toolbar
   x:= Pos (cCaption, s1);
   if x > 0 then  Caption:= s2;

   // Add a Control with the desired action and DisplayMode
   x:= Pos (cGTAction, s1);
   if x > 0 then begin
    cn:= Copy (s1, 1, x-1);
    GenButton (cn, s2, dmBoth);
   end;

   x:= Pos (cGAction, s1);
   if x > 0 then begin
    cn:= Copy (s1, 1, x-1);
    GenButton (cn, s2, dmGlyphOnly);
   end;

   x:= Pos (cTAction, s1);
   if x > 0 then begin
    cn:= Copy (s1, 1, x-1);
    GenButton (cn, s2, dmTextOnly);
   end;

   x:= Pos (cSeparator, s1);
   if x > 0 then begin // Add new Separator
    cn:= Copy (s1, 1, x-1);
    DelControl (cn);
    ts:= TToolbarSep97.Create (Self);
    ts.Name:= cn;
    InsertControl (ts);
   end;

  end; // for

  EndUpdate;

 end;
end;


procedure TFileToolbar.StoreToolbar;
begin
 Store (FOriginal);
end;


procedure TFileToolbar.ResetToolbar;
begin
 Reset (FOriginal, True);
end;


procedure TFileToolbar.LoadFromfile (const fName: string; clear: Boolean);
// Loads the Buttons etc. from the file
var ini: TInifile;
    reg: TRegIniFile;
    sl: TStringList;
begin
 if fName <> '' then begin
  if FileExists (fName) then begin
   if FUseReg then reg:= TRegInifile.Create (fName)
              else ini:= TInifile.Create (fName);
   sl:= TStringList.Create;
   try
    if FUseReg then reg.ReadSectionValues (Name, sl)
               else ini.ReadSectionValues (Name, sl);
    Reset (sl, clear);
    if Assigned (FOnLoad) then FOnLoad (Self);
   finally
    sl.Free;
    if FUseReg then reg.Free
               else ini.Free;
   end;
  end;
 end;
end;


procedure TFileToolbar.SaveToFile (const fName: string);
var ini: TInifile;
    reg: TRegIniFile;
    sl: TStringList;
    i: integer;
    s1, s2: string;
begin
 if fName <> '' then begin
  if FUseReg then  reg:= TRegInifile.Create (fName)
             else  ini:= TInifile.Create (fName);

  sl:= TStringList.Create;
  try
   Store (sl); // Store the Toolbar

   // Delete the Section, to avoid not valid keys
   if FUseReg then reg.EraseSection (Name)
              else ini.EraseSection (Name);

   for i:= 0 to sl.Count -1 do begin
    s1:= sl.Names[i];
    s2:= sl.Values[s1];
    if FUseReg then reg.WriteString (Name, s1, s2)
               else ini.WriteString (Name, s1, s2);
   end;

   if not FuseReg then ini.UpdateFile;
   if Assigned (FOnSave) then FOnSave (Self);
  finally
   sl.Free;
   if FUseReg then reg.Free
              else ini.Free;
  end;
 end;
end;


end. // FileToolbar
